@@ -8,7 +8,7 @@ Check = load_model('check', 'Check')
 
 @admin.register(Check)
 class CheckAdmin(TimeReadonlyAdminMixin, admin.ModelAdmin):
-    list_display = ['__str__', 'check', 'created', 'modified']
+    list_display = ['__str__', 'check_type', 'created', 'modified']
     search_fields = ['name', 'object_id']
     # TODO: filters
     save_on_top = True
