@@ -3,41 +3,50 @@ import logging
 import uuid
 from collections import OrderedDict
 from copy import deepcopy
+from datetime import datetime, timedelta
 from io import StringIO
 
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ObjectDoesNotExist, ValidationError
+from django.db.models import Count, Q
 from django.http import HttpResponse
+from django.utils.timezone import now
+from django.utils.translation import gettext_lazy as _
+from pytz import UTC
 from pytz import timezone as tz
 from pytz.exceptions import UnknownTimeZoneError
 from rest_framework import serializers, status
 from rest_framework.generics import GenericAPIView
-from rest_framework.permissions import BasePermission
 from rest_framework.response import Response
 from swapper import load_model
 
+from openwisp_controller.geo.api.views import (
+    DevicePermission,
+    GeoJsonLocationList,
+    LocationDeviceList,
+)
+
 from ... import settings as monitoring_settings
+from ...monitoring.configuration import ACCESS_TECHNOLOGIES
 from ...monitoring.exceptions import InvalidChartConfigException
 from ..schema import schema
 from ..signals import device_metrics_received
+from .serializers import MonitoringDeviceSerializer, MonitoringGeoJsonLocationSerializer
 
 logger = logging.getLogger(__name__)
 Chart = load_model('monitoring', 'Chart')
 Metric = load_model('monitoring', 'Metric')
 AlertSettings = load_model('monitoring', 'AlertSettings')
 Device = load_model('config', 'Device')
+DeviceMonitoring = load_model('device_monitoring', 'DeviceMonitoring')
 DeviceData = load_model('device_monitoring', 'DeviceData')
-
-
-class DevicePermission(BasePermission):
-    def has_object_permission(self, request, view, obj):
-        return request.query_params.get('key') == obj.key
+Location = load_model('geo', 'Location')
 
 
 class DeviceMetricView(GenericAPIView):
     model = DeviceData
-    queryset = DeviceData.objects.all()
+    queryset = DeviceData.objects.select_related('devicelocation').all()
     serializer_class = serializers.Serializer
     permission_classes = [DevicePermission]
     schema = schema
@@ -83,13 +92,21 @@ class DeviceMetricView(GenericAPIView):
             # prepare chart dict
             try:
                 chart_dict = chart.read(time=time, x_axys=x_axys, timezone=timezone)
+                if not chart_dict['traces']:
+                    continue
                 chart_dict['description'] = chart.description
-                chart_dict['title'] = chart.title.format(metric=chart.metric)
+                chart_dict['title'] = chart.title.format(
+                    metric=chart.metric, **chart.metric.tags
+                )
                 chart_dict['type'] = chart.type
                 chart_dict['unit'] = chart.unit
                 chart_dict['summary_labels'] = chart.summary_labels
                 chart_dict['colors'] = chart.colors
                 chart_dict['colorscale'] = chart.colorscale
+                for attr in ['fill', 'xaxis', 'yaxis']:
+                    value = getattr(chart, attr)
+                    if value:
+                        chart_dict[attr] = value
             except InvalidChartConfigException:
                 logger.exception(f'Skipped chart for metric {chart.metric}')
                 continue
@@ -154,14 +171,28 @@ class DeviceMetricView(GenericAPIView):
         except ValidationError as e:
             logger.info(e.message)
             return Response(e.message, status=status.HTTP_400_BAD_REQUEST)
+        time_obj = request.query_params.get(
+            'time', now().utcnow().strftime('%d-%m-%Y_%H:%M:%S.%f')
+        )
+        current = request.query_params.get('current', False)
+        try:
+            time = datetime.strptime(time_obj, '%d-%m-%Y_%H:%M:%S.%f').replace(
+                tzinfo=UTC
+            )
+        except ValueError:
+            return Response({'detail': _('Incorrect time format')}, status=400)
         try:
             # write data
-            self._write(request, self.instance.pk)
+            self._write(request, self.instance.pk, time=time)
         except ValidationError as e:
             logger.info(e.message_dict)
             return Response(e.message_dict, status=status.HTTP_400_BAD_REQUEST)
         device_metrics_received.send(
-            sender=self.model, instance=self.instance, request=request
+            sender=self.model,
+            instance=self.instance,
+            request=request,
+            time=time,
+            current=current,
         )
         return Response(None)
 
@@ -178,7 +209,7 @@ class DeviceMetricView(GenericAPIView):
             data['interfaces_dict'][interface['name']] = interface
         self._previous_data = data
 
-    def _write(self, request, pk):
+    def _write(self, request, pk, time=None, current=False):
         """
         write metrics to database
         """
@@ -186,8 +217,12 @@ class DeviceMetricView(GenericAPIView):
         self.instance.save_data()
         data = self.instance.data
         ct = ContentType.objects.get_for_model(Device)
+        device_extra_tags = self._get_extra_tags(self.instance)
         for interface in data.get('interfaces', []):
             ifname = interface['name']
+            extra_tags = Metric._sort_dict(device_extra_tags)
+            if 'mobile' in interface:
+                self._write_mobile_signal(interface, ifname, ct, pk, current, time=time)
             ifstats = interface.get('statistics', {})
             # Explicitly stated None to avoid skipping in case the stats are zero
             if (
@@ -208,9 +243,11 @@ class DeviceMetricView(GenericAPIView):
                     content_type=ct,
                     configuration='traffic',
                     name=name,
-                    key=ifname,
+                    key='traffic',
+                    main_tags={'ifname': Metric._makekey(ifname)},
+                    extra_tags=extra_tags,
                 )
-                metric.write(field_value, extra_values=extra_values)
+                metric.write(field_value, current, time=time, extra_values=extra_values)
                 if created:
                     self._create_traffic_chart(metric)
             try:
@@ -225,26 +262,129 @@ class DeviceMetricView(GenericAPIView):
                 content_type=ct,
                 configuration='clients',
                 name=name,
-                key=ifname,
+                key='wifi_clients',
+                main_tags={'ifname': Metric._makekey(ifname)},
+                extra_tags=extra_tags,
             )
+            # avoid tsdb overwrite clients
+            client_time = time
             for client in clients:
                 if 'mac' not in client:
                     continue
-                metric.write(client['mac'])
+                metric.write(client['mac'], current, time=client_time)
+                client_time += timedelta(microseconds=1)
             if created:
                 self._create_clients_chart(metric)
         if 'resources' not in data:
             return
         if 'load' in data['resources'] and 'cpus' in data['resources']:
             self._write_cpu(
-                data['resources']['load'], data['resources']['cpus'], pk, ct
+                data['resources']['load'],
+                data['resources']['cpus'],
+                pk,
+                ct,
+                current,
+                time=time,
             )
         if 'disk' in data['resources']:
-            self._write_disk(data['resources']['disk'], pk, ct)
+            self._write_disk(data['resources']['disk'], pk, ct, time=time)
         if 'memory' in data['resources']:
-            self._write_memory(data['resources']['memory'], pk, ct)
+            self._write_memory(data['resources']['memory'], pk, ct, current, time=time)
 
-    def _write_cpu(self, load, cpus, primary_key, content_type):
+    def _get_extra_tags(self, device):
+        tags = {'organization_id': str(device.organization_id)}
+        try:
+            device_location = device.devicelocation
+        except ObjectDoesNotExist:
+            pass
+        else:
+            tags['location_id'] = str(device_location.location_id)
+            if device_location.floorplan_id:
+                tags['floorplan_id'] = str(device_location.floorplan_id)
+        return tags
+
+    def _get_mobile_signal_type(self, signal):
+        # if only one access technology in use, return that
+        sections = list(signal.keys())
+        if len(sections) == 1:
+            return sections[0]
+        # if multiple access technologies are in use,
+        # return the most advanced one
+        access_technologies = list(ACCESS_TECHNOLOGIES.keys())
+        access_technologies.reverse()
+        for tech in access_technologies:
+            if tech in signal:
+                return tech
+
+    def _write_mobile_signal(self, interface, ifname, ct, pk, current=False, time=None):
+        access_type = self._get_mobile_signal_type(interface['mobile']['signal'])
+        data = interface['mobile']['signal'][access_type]
+        signal_power = signal_strength = None
+        extra_values = {}
+        if 'rssi' in data:
+            signal_strength = data['rssi']
+        if 'rsrp' in data:
+            signal_power = data['rsrp']
+        elif 'rscp' in data:
+            signal_power = data['rscp']
+        if signal_power is not None:
+            extra_values = {'signal_power': float(signal_power)}
+        if signal_strength is not None:
+            signal_strength = float(signal_strength)
+        if signal_strength is not None or signal_power is not None:
+            metric, created = Metric._get_or_create(
+                object_id=pk,
+                content_type=ct,
+                configuration='signal_strength',
+                name='signal strength',
+                key=ifname,
+            )
+            metric.write(signal_strength, current, time=time, extra_values=extra_values)
+            if created:
+                self._create_signal_strength_chart(metric)
+
+        snr = signal_quality = None
+        extra_values = {}
+        if 'snr' in data:
+            snr = data['snr']
+        if 'rsrq' in data:
+            signal_quality = data['rsrq']
+        if 'ecio' in data:
+            snr = data['ecio']
+        if 'sinr' in data:
+            snr = data['sinr']
+        if snr is not None:
+            extra_values = {'snr': float(snr)}
+        if signal_quality is not None:
+            signal_quality = float(signal_quality)
+        if snr is not None or signal_quality is not None:
+            metric, created = Metric._get_or_create(
+                object_id=pk,
+                content_type=ct,
+                configuration='signal_quality',
+                name='signal quality',
+                key=ifname,
+            )
+            metric.write(signal_quality, current, time=time, extra_values=extra_values)
+            if created:
+                self._create_signal_quality_chart(metric)
+        # create access technology chart
+        metric, created = Metric._get_or_create(
+            object_id=pk,
+            content_type=ct,
+            configuration='access_tech',
+            name='access technology',
+            key=ifname,
+        )
+        metric.write(
+            list(ACCESS_TECHNOLOGIES.keys()).index(access_type), current, time=time
+        )
+        if created:
+            self._create_access_tech_chart(metric)
+
+    def _write_cpu(
+        self, load, cpus, primary_key, content_type, current=False, time=None
+    ):
         extra_values = {
             'load_1': float(load[0]),
             'load_5': float(load[1]),
@@ -256,9 +396,13 @@ class DeviceMetricView(GenericAPIView):
         if created:
             self._create_resources_chart(metric, resource='cpu')
             self._create_resources_alert_settings(metric, resource='cpu')
-        metric.write(100 * float(load[0] / cpus), extra_values=extra_values)
+        metric.write(
+            100 * float(load[0] / cpus), current, time=time, extra_values=extra_values
+        )
 
-    def _write_disk(self, disk_list, primary_key, content_type):
+    def _write_disk(
+        self, disk_list, primary_key, content_type, current=False, time=None
+    ):
         used_bytes, size_bytes, available_bytes = 0, 0, 0
         for disk in disk_list:
             used_bytes += disk['used_bytes']
@@ -270,9 +414,11 @@ class DeviceMetricView(GenericAPIView):
         if created:
             self._create_resources_chart(metric, resource='disk')
             self._create_resources_alert_settings(metric, resource='disk')
-        metric.write(100 * used_bytes / size_bytes)
+        metric.write(100 * used_bytes / size_bytes, current, time=time)
 
-    def _write_memory(self, memory, primary_key, content_type):
+    def _write_memory(
+        self, memory, primary_key, content_type, current=False, time=None
+    ):
         extra_values = {
             'total_memory': memory['total'],
             'free_memory': memory['free'],
@@ -297,7 +443,7 @@ class DeviceMetricView(GenericAPIView):
         if created:
             self._create_resources_chart(metric, resource='memory')
             self._create_resources_alert_settings(metric, resource='memory')
-        metric.write(percent_used, extra_values=extra_values)
+        metric.write(percent_used, current, time=time, extra_values=extra_values)
 
     def _calculate_increment(self, ifname, stat, value):
         """
@@ -355,5 +501,60 @@ class DeviceMetricView(GenericAPIView):
         alert_settings.full_clean()
         alert_settings.save()
 
+    def _create_signal_strength_chart(self, metric):
+        chart = Chart(metric=metric, configuration='signal_strength')
+        chart.full_clean()
+        chart.save()
+
+    def _create_signal_quality_chart(self, metric):
+        chart = Chart(metric=metric, configuration='signal_quality')
+        chart.full_clean()
+        chart.save()
+
+    def _create_access_tech_chart(self, metric):
+        chart = Chart(metric=metric, configuration='access_tech')
+        chart.full_clean()
+        chart.save()
+
 
 device_metric = DeviceMetricView.as_view()
+
+
+class MonitoringGeoJsonLocationList(GeoJsonLocationList):
+    serializer_class = MonitoringGeoJsonLocationSerializer
+    queryset = (
+        Location.objects.filter(devicelocation__isnull=False)
+        .annotate(
+            device_count=Count('devicelocation'),
+            ok_count=Count(
+                'devicelocation',
+                filter=Q(devicelocation__content_object__monitoring__status='ok'),
+            ),
+            problem_count=Count(
+                'devicelocation',
+                filter=Q(devicelocation__content_object__monitoring__status='problem'),
+            ),
+            critical_count=Count(
+                'devicelocation',
+                filter=Q(devicelocation__content_object__monitoring__status='critical'),
+            ),
+            unknown_count=Count(
+                'devicelocation',
+                filter=Q(devicelocation__content_object__monitoring__status='unknown'),
+            ),
+        )
+        .order_by('-created')
+    )
+
+
+monitoring_geojson_location_list = MonitoringGeoJsonLocationList.as_view()
+
+
+class MonitoringLocationDeviceList(LocationDeviceList):
+    serializer_class = MonitoringDeviceSerializer
+
+    def get_queryset(self):
+        return super().get_queryset().select_related('monitoring').order_by('name')
+
+
+monitoring_location_device_list = MonitoringLocationDeviceList.as_view()
