@@ -1,3 +1,5 @@
+from collections import OrderedDict
+
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.translation import gettext_lazy as _
 from openwisp_notifications.types import (
@@ -23,6 +25,26 @@ DEFAULT_COLORS = [
     '#17becf',  # blue-teal
 ]
 
+ACCESS_TECHNOLOGIES = OrderedDict(
+    (
+        ('gsm', '#b42a0c'),
+        ('cdma1x', '#dd5817'),
+        ('evdo', '#df7514'),
+        ('umts', '#efdd50'),
+        ('lte', '#67c368'),
+        ('5g', '#377873'),
+    )
+)
+
+
+def _get_access_tech():
+    res = []
+    keys_list = list(ACCESS_TECHNOLOGIES.keys())
+    for i in reversed(keys_list):
+        res.append([keys_list.index(i), ACCESS_TECHNOLOGIES[i], i])
+    return res
+
+
 # under discussion
 DEFAULT_METRICS = {
     'ping': {
@@ -47,10 +69,18 @@ DEFAULT_METRICS = {
                     'max': 100,
                     'min': 0,
                     'label': _('Reachable'),
-                    'scale': [[0, '#c13000'], [0.5, '#deed0e'], [1, '#7db201']],
+                    'scale': [
+                        [0, '#c13000'],
+                        [0.1, '#ec7204'],
+                        [0.5, '#deed0e'],
+                        [0.9, '#90d000'],
+                        [1, '#7db201'],
+                    ],
                     'map': [
                         [100, '#7db201', _('Reachable')],
-                        [33, '#deed0e', _('Partly reachable')],
+                        [90, '#90d000', _('Mostly Reachable')],
+                        [50, '#deed0e', _('Partly Reachable')],
+                        [10, '#ec7204', _('Mostly Unreachable')],
                         [None, '#c13000', _('Unreachable')],
                     ],
                     'fixed_value': 100,
@@ -158,11 +188,11 @@ DEFAULT_METRICS = {
         'charts': {
             'traffic': {
                 'type': 'scatter',
-                'title': _('Traffic: {metric.key}'),
+                'title': _('Traffic: {ifname}'),
                 'label': _('Traffic'),
                 'description': _(
                     'Network traffic, download and upload, measured on '
-                    'the interface "{metric.key}", measured in GB.'
+                    'the interface "{ifname}", measured in GB.'
                 ),
                 'summary_labels': [
                     _('Total download traffic'),
@@ -183,9 +213,9 @@ DEFAULT_METRICS = {
             'wifi_clients': {
                 'type': 'bar',
                 'label': _('WiFi clients'),
-                'title': _('WiFi clients: {metric.key}'),
+                'title': _('WiFi clients: {ifname}'),
                 'description': _(
-                    'WiFi clients associated to the wireless interface "{metric.key}".'
+                    'WiFi clients associated to the wireless interface "{ifname}".'
                 ),
                 'summary_labels': [_('Total Unique WiFi clients')],
                 'unit': '',
@@ -213,7 +243,7 @@ DEFAULT_METRICS = {
                 'query': chart_query['disk'],
             }
         },
-        'alert_settings': {'operator': '>', 'threshold': 80, 'tolerance': 0},
+        'alert_settings': {'operator': '>', 'threshold': 90, 'tolerance': 0},
         'notification': {
             'problem': {
                 'verbose_name': 'Disk usage PROBLEM',
@@ -268,7 +298,7 @@ DEFAULT_METRICS = {
                 'query': chart_query['memory'],
             }
         },
-        'alert_settings': {'operator': '>', 'threshold': 95, 'tolerance': 0},
+        'alert_settings': {'operator': '>', 'threshold': 95, 'tolerance': 5},
         'notification': {
             'problem': {
                 'verbose_name': 'Memory usage PROBLEM',
@@ -318,7 +348,7 @@ DEFAULT_METRICS = {
                 'query': chart_query['cpu'],
             }
         },
-        'alert_settings': {'operator': '>', 'threshold': 90, 'tolerance': 0},
+        'alert_settings': {'operator': '>', 'threshold': 90, 'tolerance': 5},
         'notification': {
             'problem': {
                 'verbose_name': 'CPU usage PROBLEM',
@@ -347,6 +377,93 @@ DEFAULT_METRICS = {
             },
         },
     },
+    # modem-manager charts
+    'signal_strength': {
+        'label': _('Signal Strength'),
+        'name': '{name}',
+        'key': '{key}',
+        'field_name': 'signal_strength',
+        'related_fields': ['signal_power'],
+        'charts': {
+            'signal_strength': {
+                'type': 'scatter',
+                'fill': 'none',
+                'yaxis': {'zeroline': False},
+                'title': _('Signal Strength'),
+                'colors': (DEFAULT_COLORS[3], DEFAULT_COLORS[0]),
+                'description': _('Signal Strength and Signal Power, measured in dBm.'),
+                'summary_labels': [
+                    _('Average Signal Power'),
+                    _('Average Signal Strength'),
+                ],
+                'unit': _(' dBm'),
+                'order': 205,
+                'query': chart_query['signal_strength'],
+            }
+        },
+    },
+    'signal_quality': {
+        'label': _('Signal Quality'),
+        'name': '{name}',
+        'key': '{key}',
+        'field_name': 'signal_quality',
+        'related_fields': ['snr'],
+        'charts': {
+            'signal_quality': {
+                'type': 'scatter',
+                'fill': 'none',
+                'yaxis': {'zeroline': False},
+                'title': _('Signal Quality'),
+                'colors': (DEFAULT_COLORS[3], DEFAULT_COLORS[0]),
+                'description': _(
+                    _('Signal Quality and Signal to Noise Ratio (SNR), measured in dB.')
+                ),
+                'summary_labels': [
+                    _('Average Signal Quality'),
+                    _('Average Signal to Noise Ratio'),
+                ],
+                'unit': _(' dB'),
+                'order': 206,
+                'query': chart_query['signal_quality'],
+            }
+        },
+    },
+    'access_tech': {
+        'label': _('Access Technology'),
+        'name': '{name}',
+        'key': '{key}',
+        'field_name': 'access_tech',
+        'charts': {
+            'access_tech': {
+                'type': 'bar',
+                'title': _('Access Technology'),
+                'description': _(
+                    _(
+                        'Shows the access technology (LTE, UTMS, CDMA1x, etc.) '
+                        'in use on the device at the specified time.'
+                    )
+                ),
+                'order': 207,
+                'unit': '',
+                'colorscale': {
+                    'label': '',
+                    'map': _get_access_tech(),
+                    'max': 5,
+                    'min': 0,
+                    'scale': (
+                        (0, '#b42a0c'),
+                        (0.1, '#dd5817'),
+                        (0.2, '#df7514'),
+                        (0.5, '#efdd50'),
+                        (0.8, '#67c368'),
+                        (1, '#377873'),
+                    ),
+                    'fixed_value': 100,
+                },
+                'query': chart_query['access_tech'],
+            }
+        },
+    },
 }
 
 DEFAULT_CHARTS = {}
