@@ -1,14 +1,13 @@
 from datetime import timedelta
-from unittest.mock import patch
 
-from django.core import mail
-from django.urls import reverse
 from django.utils import timezone
-from django.utils.html import strip_tags
 from freezegun import freeze_time
 from swapper import load_model
 
-from ...device.tests import DeviceMonitoringTestCase
+from ...device.tests import (
+    DeviceMonitoringTestCase,
+    DeviceMonitoringTransactionTestcase,
+)
 
 Metric = load_model('monitoring', 'Metric')
 Notification = load_model('openwisp_notifications', 'Notification')
@@ -36,7 +35,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         with self.subTest('Test notification for metric exceeding alert settings'):
             m.write(99)
             m.refresh_from_db()
-            self.assertFalse(m.is_healthy)
+            self.assertEqual(m.is_healthy, False)
+            self.assertEqual(m.is_healthy_tolerant, False)
             self.assertEqual(Notification.objects.count(), 1)
             n = notification_queryset.first()
             self.assertEqual(n.recipient, admin)
@@ -47,13 +47,15 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         with self.subTest('Test no double alarm for metric exceeding alert settings'):
             m.write(95)
             m.refresh_from_db()
-            self.assertFalse(m.is_healthy)
+            self.assertEqual(m.is_healthy, False)
+            self.assertEqual(m.is_healthy_tolerant, False)
             self.assertEqual(Notification.objects.count(), 1)
 
         with self.subTest('Test notification for metric falling behind alert settings'):
             m.write(60)
             m.refresh_from_db()
-            self.assertTrue(m.is_healthy)
+            self.assertEqual(m.is_healthy, True)
+            self.assertEqual(m.is_healthy_tolerant, True)
             self.assertEqual(Notification.objects.count(), 2)
             n = notification_queryset.last()
             self.assertEqual(n.recipient, admin)
@@ -66,7 +68,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         ):
             m.write(40)
             m.refresh_from_db()
-            self.assertTrue(m.is_healthy)
+            self.assertEqual(m.is_healthy, True)
+            self.assertEqual(m.is_healthy_tolerant, True)
             self.assertEqual(Notification.objects.count(), 2)
 
     def test_general_check_threshold_crossed_deferred(self):
@@ -77,7 +80,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         )
         m.write(99, time=ten_minutes_ago)
         m.refresh_from_db()
-        self.assertFalse(m.is_healthy)
+        self.assertEqual(m.is_healthy, False)
+        self.assertEqual(m.is_healthy_tolerant, False)
         self.assertEqual(Notification.objects.count(), 1)
         n = notification_queryset.first()
         self.assertEqual(n.recipient, admin)
@@ -92,7 +96,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
             metric=m, custom_operator='>', custom_threshold=90, custom_tolerance=1
         )
         m.write(99)
-        self.assertTrue(m.is_healthy)
+        self.assertEqual(m.is_healthy, True)
+        self.assertEqual(m.is_healthy_tolerant, True)
         self.assertEqual(Notification.objects.count(), 0)
 
     def test_resources_metric_threshold_deferred_not_crossed(self):
@@ -103,20 +108,26 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
 
     def test_cpu_metric_threshold_crossed(self):
         admin = self._create_admin()
-        test_data_path = (
-            'openwisp_monitoring.device.tests.DeviceMonitoringTestCase._data'
-        )
+        org = self._create_org()
+        device = self._create_device(organization=org)
+        # creates metric and alert settings
         data = self._data()
         data['resources']['load'] = [0.99, 0.99, 0.99]
-        with patch(test_data_path, return_value=data):
-            self.create_test_data()
-            m = Metric.objects.get(name='CPU usage')
-            self.assertEqual(Notification.objects.count(), 1)
-            n = Notification.objects.first()
-            self.assertEqual(n.recipient, admin)
-            self.assertEqual(n.actor, m)
-            self.assertEqual(n.action_object, m.alertsettings)
-            self.assertEqual(n.level, 'warning')
+        response = self._post_data(device.id, device.key, data)
+        self.assertEqual(response.status_code, 200)
+        # retrieve created metric
+        metric = Metric.objects.get(name='CPU usage')
+        # simplify test by setting tolerance to 0
+        metric.alertsettings.custom_tolerance = 0
+        metric.alertsettings.save()
+        # trigger alert
+        metric.write(99.0)
+        self.assertEqual(Notification.objects.count(), 1)
+        n = Notification.objects.first()
+        self.assertEqual(n.recipient, admin)
+        self.assertEqual(n.actor, metric)
+        self.assertEqual(n.action_object, metric.alertsettings)
+        self.assertEqual(n.level, 'warning')
 
     def test_general_check_threshold_crossed_for_long_time(self):
         """
@@ -135,7 +146,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         with self.subTest('Test no notification is generated for healthy status'):
             m.write(89, time=ten_minutes_ago)
             m.refresh_from_db()
-            self.assertTrue(m.is_healthy)
+            self.assertEqual(m.is_healthy, True)
+            self.assertEqual(m.is_healthy_tolerant, True)
             self.assertEqual(Notification.objects.count(), 0)
 
         with self.subTest('Test no notification is generated when check=False'):
@@ -145,7 +157,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         with self.subTest('Test notification for metric with current timestamp'):
             m.write(92)
             m.refresh_from_db()
-            self.assertFalse(m.is_healthy)
+            self.assertEqual(m.is_healthy, False)
+            self.assertEqual(m.is_healthy_tolerant, False)
             self.assertEqual(Notification.objects.count(), 1)
             n = notification_queryset.first()
             self.assertEqual(n.recipient, admin)
@@ -156,21 +169,24 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         with self.subTest('Test no recovery notification yet (tolerance not passed)'):
             m.write(50)
             m.refresh_from_db()
-            self.assertFalse(m.is_healthy)
+            self.assertEqual(m.is_healthy, True)
+            self.assertEqual(m.is_healthy_tolerant, False)
             self.assertEqual(Notification.objects.count(), 1)
 
         with self.subTest('Tolerance still not passed, not expecting a recovery yet'):
             with freeze_time(start_time + timedelta(minutes=2)):
                 m.write(51)
             m.refresh_from_db()
-            self.assertFalse(m.is_healthy)
+            self.assertEqual(m.is_healthy, True)
+            self.assertEqual(m.is_healthy_tolerant, False)
             self.assertEqual(Notification.objects.count(), 1)
 
         with self.subTest('Test recovery notification after tolerance is passed'):
             with freeze_time(ten_minutes_after):
                 m.write(50)
             m.refresh_from_db()
-            self.assertTrue(m.is_healthy)
+            self.assertEqual(m.is_healthy, True)
+            self.assertEqual(m.is_healthy_tolerant, True)
             self.assertEqual(Notification.objects.count(), 2)
             n = notification_queryset.last()
             self.assertEqual(n.recipient, admin)
@@ -190,7 +206,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         ):
             om.write(99)
             om.refresh_from_db()
-            self.assertFalse(om.is_healthy)
+            self.assertEqual(om.is_healthy, False)
+            self.assertEqual(om.is_healthy_tolerant, False)
             self.assertEqual(Notification.objects.count(), 1)
             n = notification_queryset.first()
             self.assertEqual(n.recipient, admin)
@@ -204,7 +221,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         ):
             om.write(95)
             om.refresh_from_db()
-            self.assertFalse(om.is_healthy)
+            self.assertEqual(om.is_healthy, False)
+            self.assertEqual(om.is_healthy_tolerant, False)
             self.assertEqual(Notification.objects.count(), 1)
 
         with self.subTest(
@@ -212,7 +230,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         ):
             om.write(60)
             om.refresh_from_db()
-            self.assertTrue(om.is_healthy)
+            self.assertEqual(om.is_healthy, True)
+            self.assertEqual(om.is_healthy_tolerant, True)
             self.assertEqual(Notification.objects.count(), 2)
             n = notification_queryset.last()
             self.assertEqual(n.recipient, admin)
@@ -226,7 +245,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         ):
             om.write(40)
             om.refresh_from_db()
-            self.assertTrue(om.is_healthy)
+            self.assertEqual(om.is_healthy, True)
+            self.assertEqual(om.is_healthy_tolerant, True)
             self.assertEqual(Notification.objects.count(), 2)
 
     def test_object_check_threshold_crossed_deferred(self):
@@ -237,7 +257,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         )
         om.write(99, time=ten_minutes_ago)
         om.refresh_from_db()
-        self.assertFalse(om.is_healthy)
+        self.assertEqual(om.is_healthy, False)
+        self.assertEqual(om.is_healthy_tolerant, False)
         self.assertEqual(Notification.objects.count(), 1)
         n = notification_queryset.first()
         self.assertEqual(n.recipient, admin)
@@ -254,7 +275,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         )
         om.write(99)
         om.refresh_from_db()
-        self.assertTrue(om.is_healthy)
+        self.assertEqual(om.is_healthy, False)
+        self.assertEqual(om.is_healthy_tolerant, True)
         self.assertEqual(Notification.objects.count(), 0)
 
     def test_object_check_threshold_crossed_for_long_time(self):
@@ -269,7 +291,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         self.assertEqual(Notification.objects.count(), 0)
         om.write(92)
         om.refresh_from_db()
-        self.assertFalse(om.is_healthy)
+        self.assertEqual(om.is_healthy, False)
+        self.assertEqual(om.is_healthy_tolerant, False)
         self.assertEqual(Notification.objects.count(), 1)
         n = notification_queryset.first()
         self.assertEqual(n.recipient, admin)
@@ -280,18 +303,21 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         # ensure double alarm not sent
         om.write(95)
         om.refresh_from_db()
-        self.assertFalse(om.is_healthy)
+        self.assertEqual(om.is_healthy, False)
+        self.assertEqual(om.is_healthy_tolerant, False)
         self.assertEqual(Notification.objects.count(), 1)
         # value back to normal but tolerance not passed yet
         om.write(60)
         om.refresh_from_db()
-        self.assertFalse(om.is_healthy)
+        self.assertEqual(om.is_healthy, True)
+        self.assertEqual(om.is_healthy_tolerant, False)
         self.assertEqual(Notification.objects.count(), 1)
         # tolerance passed
         with freeze_time(ten_minutes_after):
             om.write(60)
         om.refresh_from_db()
-        self.assertTrue(om.is_healthy)
+        self.assertEqual(om.is_healthy, True)
+        self.assertEqual(om.is_healthy_tolerant, True)
         self.assertEqual(Notification.objects.count(), 2)
         n = notification_queryset.last()
         self.assertEqual(n.recipient, admin)
@@ -303,7 +329,8 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
         with freeze_time(ten_minutes_after + timedelta(minutes=5)):
             om.write(40)
         om.refresh_from_db()
-        self.assertTrue(om.is_healthy)
+        self.assertEqual(om.is_healthy, True)
+        self.assertEqual(om.is_healthy_tolerant, True)
         self.assertEqual(Notification.objects.count(), 2)
 
     def test_flapping_metric_with_tolerance(self):
@@ -316,31 +343,100 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
             om.write(1)
             self.assertEqual(Notification.objects.count(), 0)
             om.refresh_from_db()
-            self.assertTrue(om.is_healthy)
+            self.assertEqual(om.is_healthy, True)
+            self.assertEqual(om.is_healthy_tolerant, True)
         with freeze_time(start_time - timedelta(minutes=20)):
             om.write(0)
             self.assertEqual(Notification.objects.count(), 0)
             om.refresh_from_db()
-            self.assertTrue(om.is_healthy)
+            self.assertEqual(om.is_healthy, False)
+            self.assertEqual(om.is_healthy_tolerant, True)
         with freeze_time(start_time - timedelta(minutes=15)):
             om.write(1)
             self.assertEqual(Notification.objects.count(), 0)
             om.refresh_from_db()
-            self.assertTrue(om.is_healthy)
+            self.assertEqual(om.is_healthy, True)
+            self.assertEqual(om.is_healthy_tolerant, True)
         with freeze_time(start_time - timedelta(minutes=10)):
             om.write(0)
             self.assertEqual(Notification.objects.count(), 0)
             om.refresh_from_db()
-            self.assertTrue(om.is_healthy)
+            self.assertEqual(om.is_healthy, False)
+            self.assertEqual(om.is_healthy_tolerant, True)
         with freeze_time(start_time - timedelta(minutes=5)):
             om.write(0)
             self.assertEqual(Notification.objects.count(), 0)
             om.refresh_from_db()
-            self.assertTrue(om.is_healthy)
+            self.assertEqual(om.is_healthy, False)
+            self.assertEqual(om.is_healthy_tolerant, True)
         om.write(0)
         self.assertEqual(Notification.objects.count(), 1)
         om.refresh_from_db()
-        self.assertFalse(om.is_healthy)
+        self.assertEqual(om.is_healthy, False)
+        self.assertEqual(om.is_healthy_tolerant, False)
+
+    def test_notification_types(self):
+        self._create_admin()
+        m = self._create_object_metric(name='load')
+        self._create_alert_settings(
+            metric=m, custom_operator='>', custom_threshold=90, custom_tolerance=0
+        )
+        exp_message = (
+            '{n.actor.name} for device '
+            '<a href="https://example.com/admin/openwisp_users/user/{n.target.id}/change/">tester</a>'
+            ' {n.verb}.'
+        )
+        with self.subTest("Test notification for 'alert settings crossed'"):
+            m.write(99)
+            n = notification_queryset.first()
+            self.assertEqual(n.level, 'warning')
+            self.assertEqual(n.verb, 'crossed the threshold')
+            self.assertEqual(
+                n.email_subject, f'[example.com] PROBLEM: {n.actor.name} {n.target}'
+            )
+            self.assertIn(exp_message.format(n=n), n.message)
+
+        with self.subTest("Test notification for 'under alert settings'"):
+            m.write(80)
+            n = notification_queryset.last()
+            self.assertEqual(n.level, 'info')
+            self.assertEqual(n.verb, 'returned within the threshold')
+            self.assertEqual(
+                n.email_subject, f'[example.com] RECOVERY: {n.actor.name} {n.target}'
+            )
+            self.assertIn(exp_message.format(n=n), n.message)
+
+    def test_alerts_disabled(self):
+        self._create_admin()
+        d = self._create_device(organization=self._get_org())
+        m = self._create_general_metric(name='load', content_object=d)
+        self._create_alert_settings(
+            metric=m,
+            custom_operator='>',
+            custom_threshold=90,
+            custom_tolerance=1,
+            is_active=False,
+        )
+        m.write(99, time=ten_minutes_ago)
+        m.refresh_from_db()
+        self.assertEqual(m.is_healthy, False)
+        self.assertEqual(m.is_healthy_tolerant, False)
+        d.refresh_from_db()
+        self.assertEqual(d.monitoring.status, 'problem')
+        self.assertEqual(Notification.objects.count(), 0)
+
+
+class TestTransactionMonitoringNotifications(DeviceMonitoringTransactionTestcase):
+    device_model = Device
+    config_model = Config
+
+    def _check_notification_parameters(self, notification, recepient, metric, target):
+        self.assertEqual(notification.recipient, recepient)
+        self.assertEqual(notification.actor, metric)
+        self.assertEqual(notification.target, target)
+        self.assertEqual(notification.action_object, metric.alertsettings)
+        self.assertEqual(notification.level, 'warning')
+        self.assertEqual(notification.verb, 'is not reachable')
 
     def test_multiple_notifications(self):
         testorg = self._create_org()
@@ -368,10 +464,10 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
             self.assertEqual(Notification.objects.count(), 1)
             n = notification_queryset.first()
             self._check_notification_parameters(n, admin, m, None)
-            self.assertEqual(
-                '<p>The device <a href="#">None</a> is not reachable.</p>', n.message
+            self.assertIn(
+                'The device <a href="#">None</a> is not reachable.', n.message
             )
-            Notification.objects.all().delete()
+        Notification.objects.all().delete()
 
         with self.subTest('Test object metric multiple notifications'):
             d = self._create_device(organization=testorg)
@@ -387,7 +483,7 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
             self.assertIn('is not reachable.', n.message)
             n = notification_queryset.last()
             self._check_notification_parameters(n, staff, om, d)
-            Notification.objects.all().delete()
+        Notification.objects.all().delete()
 
         with self.subTest('Test object metric multiple notifications no org'):
             om = self._create_object_metric(name='logins', content_object=user)
@@ -399,112 +495,3 @@ class TestMonitoringNotifications(DeviceMonitoringTestCase):
             self.assertEqual(Notification.objects.count(), 1)
             n = notification_queryset.first()
             self._check_notification_parameters(n, admin, om, user)
-
-    def _check_notification_parameters(self, notification, recepient, metric, target):
-        self.assertEqual(notification.recipient, recepient)
-        self.assertEqual(notification.actor, metric)
-        self.assertEqual(notification.target, target)
-        self.assertEqual(notification.action_object, metric.alertsettings)
-        self.assertEqual(notification.level, 'warning')
-        self.assertEqual(notification.verb, 'is not reachable')
-
-    def test_email_notification(self):
-        self._create_admin()
-        d = self._create_device(organization=self._get_org())
-        m = self._create_general_metric(name='load', content_object=d)
-        self._create_alert_settings(
-            metric=m, custom_operator='>', custom_threshold=90, custom_tolerance=0
-        )
-        exp_target_link = f'https://example.com/admin/config/device/{d.id}/change/'
-        exp_email_body = '{message}\n\nFor more information see {email_link}.'
-
-        with self.subTest('Test notification email for metric crossed alert settings'):
-            m.write(99)
-            n = notification_queryset.first()
-            url_path = reverse('notifications:notification_read_redirect', args=[n.pk])
-            email_link = f'https://example.com{url_path}'
-            email = mail.outbox.pop()
-            html_message, content_type = email.alternatives.pop()
-            self.assertEqual(email.subject, n.email_subject)
-            self.assertEqual(
-                email.body,
-                exp_email_body.format(
-                    message=strip_tags(n.message), email_link=email_link
-                ),
-            )
-            self.assertIn(
-                f'<a href="{email_link}">'
-                'For further information see "device: default.test.device".</a>',
-                html_message,
-            )
-            self.assertIn(exp_target_link, n.message)
-
-        with self.subTest('Test notification email for metric returned under threhold'):
-            m.write(50)
-            n = notification_queryset.last()
-            url_path = reverse('notifications:notification_read_redirect', args=[n.pk])
-            email_link = f'https://example.com{url_path}'
-            email = mail.outbox.pop()
-            html_message, content_type = email.alternatives.pop()
-            self.assertEqual(email.subject, n.email_subject)
-            self.assertEqual(
-                email.body,
-                exp_email_body.format(
-                    message=strip_tags(n.message), email_link=email_link
-                ),
-            )
-            self.assertIn(
-                f'<a href="{email_link}">'
-                'For further information see "device: default.test.device".</a>',
-                html_message,
-            )
-            self.assertIn(exp_target_link, n.message)
-
-    def test_notification_types(self):
-        self._create_admin()
-        m = self._create_object_metric(name='load')
-        self._create_alert_settings(
-            metric=m, custom_operator='>', custom_threshold=90, custom_tolerance=0
-        )
-        exp_message = (
-            '<p>{n.actor.name} for device '
-            '<a href="https://example.com/admin/openwisp_users/user/{n.target.id}/change/">tester</a>'
-            ' {n.verb}.</p>'
-        )
-        with self.subTest("Test notification for 'alert settings crossed'"):
-            m.write(99)
-            n = notification_queryset.first()
-            self.assertEqual(n.level, 'warning')
-            self.assertEqual(n.verb, 'crossed the threshold')
-            self.assertEqual(
-                n.email_subject, f'[example.com] PROBLEM: {n.actor.name} {n.target}'
-            )
-            self.assertEqual(n.message, exp_message.format(n=n))
-
-        with self.subTest("Test notification for 'under alert settings'"):
-            m.write(80)
-            n = notification_queryset.last()
-            self.assertEqual(n.level, 'info')
-            self.assertEqual(n.verb, 'returned within the threshold')
-            self.assertEqual(
-                n.email_subject, f'[example.com] RECOVERY: {n.actor.name} {n.target}'
-            )
-            self.assertEqual(n.message, exp_message.format(n=n))
-
-    def test_alerts_disabled(self):
-        self._create_admin()
-        d = self._create_device(organization=self._get_org())
-        m = self._create_general_metric(name='load', content_object=d)
-        self._create_alert_settings(
-            metric=m,
-            custom_operator='>',
-            custom_threshold=90,
-            custom_tolerance=1,
-            is_active=False,
-        )
-        m.write(99, time=ten_minutes_ago)
-        m.refresh_from_db()
-        self.assertFalse(m.is_healthy)
-        d.refresh_from_db()
-        self.assertEqual(d.monitoring.status, 'problem')
-        self.assertEqual(Notification.objects.count(), 0)
